@@ -69,12 +69,16 @@ __date__ = "16/01/2020"
 import argparse
 import logging
 
-from nxtomomill.converter.hdf5.acquisition.baseacquisition import SourceType
+from nxtomomill.nexus.nxsource import SourceType
 from nxtomomill import utils
 from nxtomomill import converter
 from nxtomomill.utils import Progress
 from nxtomomill.settings import Tomo
-from tomoscan.esrf.edfscan import EDFTomoScan
+
+try:
+    from tomoscan.esrf.scan.edfscan import EDFTomoScan
+except ImportError:
+    from tomoscan.esrf.edfscan import EDFTomoScan
 
 EDF_MOTOR_POS = Tomo.EDF.MOTOR_POS
 EDF_MOTOR_MNE = Tomo.EDF.MOTOR_MNE
