@@ -296,6 +296,12 @@ def main(argv):
         default=None,
         help="Titles corresponding to zserie init scans",
     )
+    parser.add_argument(
+        "--init_pcotomo_titles",
+        "--init-pcotomo-titles",
+        default=None,
+        help="Titles corresponding to pcotomo init scans",
+    )
     parser.add_argument(
         "--dark_titles",
         "--dark-titles",
