@@ -215,7 +215,16 @@ def main(argv):
         action="store_false",
         default=None,
     )
-
+    parser.add_argument(
+        "--data-copy",
+        "--copy-data",
+        help="Force data duplication of frames. This will permit to have an "
+        "'all-embed' file. Otherwise the detector/data dataset will haves "
+        "links to other files.",
+        action="store_true",
+        dest="duplicate_data",
+        default=None,
+    )
     parser.add_argument(
         "--x_trans_keys",
         "--x-trans-keys",
@@ -254,6 +263,12 @@ def main(argv):
         default=None,
         help="Valid dataset name for rotation angle",
     )
+    parser.add_argument(
+        "--rocking_keys",
+        "--rocking-keys",
+        default=None,
+        help="Valid dataset name for rocking angle",
+    )
     parser.add_argument(
         "--acq_expo_time_keys",
         "--acq-expo-time-keys",
@@ -280,6 +295,12 @@ def main(argv):
         default=None,
         help="Titles corresponding to init scans",
     )
+    parser.add_argument(
+        "--init_pcotomo_titles",
+        "--init-pcotomo-titles",
+        default=None,
+        help="Titles corresponding to pcotomo init scans",
+    )
     parser.add_argument(
         "--init_zserie_titles",
         "--init-zserie-titles",
