@@ -52,7 +52,7 @@ __date__ = "23/02/2021"
 import logging
 import argparse
 from nxtomomill.io import generate_default_h5_config
-from nxtomomill.io import TomoHDF5Config
+from nxtomomill.io import TomoHDF5Config, XRD3DHDF5Config
 
 logging.basicConfig(level=logging.INFO)
 _logger = logging.getLogger(__name__)
@@ -78,15 +78,26 @@ def main(argv):
         action="store_true",
         default=False,
     )
+    parser.add_argument(
+        "--config-3dxrd",
+        help="Configuration file for 3dxrd dataset",
+        action="store_true",
+        default=False,
+    )
 
     options = parser.parse_args(argv[1:])
-    configuration = generate_default_h5_config()
-    if options.from_title_names:
-        if options.from_scan_urls:
-            raise ValueError(
-                "`from-title-names` and `from-scan-urls` are " "exclusive options"
-            )
-        del configuration[TomoHDF5Config.FRAME_TYPE_SECTION_DK]
-    elif options.from_scan_urls:
-        del configuration[TomoHDF5Config.ENTRIES_AND_TITLES_SECTION_DK]
-    TomoHDF5Config.dict_to_cfg(file_path=options.output_file, dict_=configuration)
+
+    if options.config_3dxrd:
+        configuration = generate_default_h5_config(config_3dxrd=True)
+        XRD3DHDF5Config.dict_to_cfg(file_path=options.output_file, dict_=configuration)
+    else:
+        configuration = generate_default_h5_config(config_3dxrd=False)
+        if options.from_title_names:
+            if options.from_scan_urls:
+                raise ValueError(
+                    "`from-title-names` and `from-scan-urls` are " "exclusive options"
+                )
+            del configuration[TomoHDF5Config.FRAME_TYPE_SECTION_DK]
+        elif options.from_scan_urls:
+            del configuration[TomoHDF5Config.ENTRIES_AND_TITLES_SECTION_DK]
+        TomoHDF5Config.dict_to_cfg(file_path=options.output_file, dict_=configuration)
