@@ -74,7 +74,12 @@ import logging
 from nxtomomill import utils
 from silx.io.url import DataUrl
 from silx.utils.enum import Enum as _Enum
-from tomoscan.esrf.hdf5scan import HDF5TomoScan
+
+try:
+    from tomoscan.esrf.scan.hdf5scan import HDF5TomoScan
+except ImportError:
+    from tomoscan.esrf.hdf5scan import HDF5TomoScan
+
 from nxtomomill.utils import ImageKey
 import argparse
 import os
