@@ -41,11 +41,11 @@ from silx.io.url import DataUrl
 from nxtomomill.utils import ImageKey
 from nxtomomill.converter.baseconverter import BaseConverter
 from silx.io.utils import h5py_read_dataset
-from nxtomomill.converter.hdf5.acquisition.baseacquisition import EntryReader
-from nxtomomill.converter.hdf5.acquisition.baseacquisition import DatasetReader
+from nxtomomill.utils.hdf5 import EntryReader
+from nxtomomill.utils.hdf5 import DatasetReader
 from nxtomomill.converter.version import version as converter_version
-from nxtomomill.utils import FieldOfView
-from nxtomomill.utils import _FrameAppender
+from nxtomomill.nexus.nxdetector import FieldOfView
+from nxtomomill.utils import FrameAppender
 from nxtomomill.io.config import DXFileConfiguration
 from tomoscan.unitsystem import metricsystem
 import logging
@@ -329,7 +329,7 @@ class _DxFileToNxConverter(BaseConverter):
         if self.energy is None:
             try:
                 self._configuration.energy = self._read_energy()
-            except Exception as e:
+            except Exception:
                 self._configuration.energy = None
         if self.energy is None:
             self._configuration.energy = self.DEFAULT_BEAM_ENERGY
@@ -500,7 +500,7 @@ class _DxFileToNxConverter(BaseConverter):
         try:
             with DatasetReader(self._data_darks_url) as dark_dataset:
                 n_dark = dark_dataset.shape[0]
-            _FrameAppender(
+            FrameAppender(
                 self._data_darks_url,
                 self.output_file,
                 data_path=dataset_path,
@@ -523,7 +523,7 @@ class _DxFileToNxConverter(BaseConverter):
             with DatasetReader(self._data_flats_url) as flat_dataset:
                 n_flat = flat_dataset.shape[0]
 
-            _FrameAppender(
+            FrameAppender(
                 self._data_flats_url,
                 self.output_file,
                 data_path=dataset_path,
@@ -545,14 +545,14 @@ class _DxFileToNxConverter(BaseConverter):
         try:
             with DatasetReader(self._data_proj_url) as proj_dataset:
                 n_proj = proj_dataset.shape[0]
-            _FrameAppender(
+            FrameAppender(
                 self._data_proj_url,
                 self.output_file,
                 data_path=dataset_path,
                 where="end",
                 logger=_logger,
             ).process()
-        except Exception as e:
+        except Exception:
             _logger.warning(
                 "No projections found in {} or unable to add them to the dataset".format(
                     self._data_proj_url.path()
