@@ -36,20 +36,24 @@ __date__ = "27/11/2020"
 
 from collections import namedtuple
 from typing import Optional
-from nxtomomill.converter.hdf5.acquisition.baseacquisition import SourceType
+from nxtomomill.nexus.nxsource import SourceType
 from nxtomomill import utils
 from nxtomomill.utils import ImageKey
 from nxtomomill.converter.version import version as converter_version
-from nxtomomill.converter.utils import create_nx_data_group
-from nxtomomill.converter.utils import link_nxbeam_to_root
+from nxtomomill.nexus.utils import create_nx_data_group
+from nxtomomill.nexus.utils import link_nxbeam_to_root
 from nxtomomill.settings import Tomo
-from tomoscan.esrf.edfscan import EDFTomoScan
+
+try:
+    from tomoscan.esrf.scan.edfscan import EDFTomoScan
+except ImportError:
+    from tomoscan.esrf.edfscan import EDFTomoScan
 from tomoscan.unitsystem import metricsystem
 from tomoscan.io import HDF5File
-import datetime
 import fabio
 import numpy
 import os
+import logging
 
 EDF_MOTOR_POS = Tomo.EDF.MOTOR_POS
 EDF_MOTOR_MNE = Tomo.EDF.MOTOR_MNE
@@ -61,8 +65,6 @@ EDF_X_TRANS = Tomo.EDF.X_TRANS
 EDF_Y_TRANS = Tomo.EDF.Y_TRANS
 EDF_Z_TRANS = Tomo.EDF.Z_TRANS
 
-import logging
-
 _logger = logging.getLogger(__name__)
 
 
@@ -138,6 +140,7 @@ def edf_to_nx(
 
     DARK_ACCUM_FACT = True
     with HDF5File(fileout_h5, "w") as h5d:
+        metadata = []
         proj_urls = scan.get_proj_urls(scan=scan.path)
 
         for dark_to_find in file_keys.dark_names:
@@ -181,7 +184,7 @@ def edf_to_nx(
                         .split(" ")
                         .index(file_keys.rot_angle_key)
                     )
-                except:
+                except (KeyError, ValueError):
                     rotangle_index = -1
                 try:
                     xtrans_index = (
@@ -189,7 +192,7 @@ def edf_to_nx(
                         .split(" ")
                         .index(file_keys.x_trans_key)
                     )
-                except:
+                except (KeyError, ValueError):
                     xtrans_index = -1
                 try:
                     ytrans_index = (
@@ -197,7 +200,7 @@ def edf_to_nx(
                         .split(" ")
                         .index(file_keys.y_trans_key)
                     )
-                except:
+                except (KeyError, ValueError):
                     ytrans_index = -1
                 try:
                     ztrans_index = (
@@ -205,7 +208,7 @@ def edf_to_nx(
                         .split(" ")
                         .index(file_keys.z_trans_key)
                     )
-                except:
+                except (KeyError, ValueError):
                     ztrans_index = -1
 
                 if hasattr(fid, "bytecode"):
@@ -246,9 +249,11 @@ def edf_to_nx(
         )
 
         if sample_name is None:
+            # try to deduce sample name from scan path.
             try:
                 sample_name = os.path.abspath(scan.path).split(os.sep)[-3:]
-            except:
+                sample_name = os.sep.join(sample_name)
+            except Exception:
                 sample_name = "unknow"
         h5d["/entry/sample/name"] = sample_name
         if instrument_name is not None:
@@ -363,7 +368,7 @@ def edf_to_nx(
                 else:
                     data = fabio_file.getframe(index).data
                     header = fabio_file.getframe(index).header
-            except Exception as e:
+            except Exception:
                 data = None
                 header = None
             finally:
@@ -396,6 +401,8 @@ def edf_to_nx(
                 _logger.info("ignore " + dk_url.file_path())
                 continue
             data, header = read_url(dk_url)
+            metadata.append(header)
+
             data_dataset[nf, :, :] = data * norm_dark
             keys_dataset[nf] = ImageKey.DARK_FIELD.value
             keys_control_dataset[nf] = ImageKey.DARK_FIELD.value
@@ -469,6 +476,8 @@ def edf_to_nx(
                         _logger.info("ignore " + refUrl.file_path())
                         continue
                     data, header = read_url(refUrl)
+                    metadata.append(header)
+
                     dataDataset[nfr, :, :] = data + test_val
                     keysDataset[nfr] = ImageKey.FLAT_FIELD.value
                     keysCDataset[nfr] = ImageKey.FLAT_FIELD.value
@@ -530,6 +539,7 @@ def edf_to_nx(
                 )
                 iref_pj += 1
             data, header = read_url(proj_url)
+            metadata.append(header)
 
             data_dataset[nf, :, :] = data
             keys_dataset[nf] = ImageKey.PROJECTION.value
@@ -604,6 +614,18 @@ def edf_to_nx(
 
         h5d.flush()
 
+    for i_meta, meta in enumerate(metadata):
+        # save metadata
+        from silx.io.dictdump import dicttoh5
+
+        dicttoh5(
+            meta,
+            fileout_h5,
+            h5path=f"/entry/instrument/positioners/{i_meta}",
+            update_mode="replace",
+            mode="a",
+        )
+
     try:
         create_nx_data_group(
             file_path=output_file, entry_path="entry", axis_scale=["linear", "linear"]
@@ -614,7 +636,7 @@ def edf_to_nx(
     # create beam group at root for compatibility
     try:
         link_nxbeam_to_root(file_path=output_file, entry_path="entry")
-    except Exception as e:
+    except Exception:
         pass
 
     return fileout_h5, "entry"
