@@ -33,10 +33,15 @@ __authors__ = [
 __license__ = "MIT"
 __date__ = "27/11/2020"
 
+import os
 import h5py
+from nxtomomill.nexus.nxtomo import NXtomo
+from nxtomomill.utils.hdf5 import EntryReader
+from nxtomomill.utils.utils import embed_url, get_file_name  # noqa F401
+from nxtomomill.utils.h5pyutils import from_data_url_to_virtual_source
 
 try:
-    import hdf5plugin
+    import hdf5plugin  # noqa F401
 except ImportError:
     pass
 from nxtomomill.plugins import (
@@ -47,82 +52,18 @@ from silx.io.url import DataUrl
 from nxtomomill.io.config import TomoHDF5Config
 from silx.io.utils import h5py_read_dataset
 from tomoscan.unitsystem import metricsystem
-from tomoscan.io import HDF5File
 from collections import OrderedDict
 import numpy
 import typing
 import logging
-import contextlib
-
-try:
-    from tomoscan.scanbase import SourceType
-except ImportError:
-    from silx.utils.enum import Enum as _Enum
-
-    class SourceType(_Enum):
-        SPALLATION_NEUTRON = "Spallation Neutron Source"
-        PULSED_REACTOR_NEUTRON_SOURCE = "Pulsed Reactor Neutron Source"
-        REACTOR_NEUTRON_SOURCE = "Reactor Neutron Source"
-        SYNCHROTRON_X_RAY_SOURCE = "Synchrotron X-ray Source"
-        PULSED_MUON_SOURCE = "Pulsed Muon Source"
-        ROTATING_ANODE_X_RAY = "Rotating Anode X-ray"
-        FIXED_TUBE_X_RAY = "Fixed Tube X-ray"
-        UV_LASER = "UV Laser"
-        FREE_ELECTRON_LASER = "Free-Electron Laser"
-        OPTICAL_LASER = "Optical Laser"
-        ION_SOURCE = "Ion Source"
-        UV_PLASMA_SOURCE = "UV Plasma Source"
-        METAL_JET_X_RAY = "Metal Jet X-ray"
 
 
 _logger = logging.getLogger(__name__)
 
 
-class _BaseReader(contextlib.AbstractContextManager):
-    def __init__(self, url: DataUrl):
-        if not isinstance(url, DataUrl):
-            raise TypeError("url should be an instance of DataUrl")
-        if url.scheme() not in ("silx", "h5py"):
-            raise ValueError("Valid scheme are silx and h5py")
-        if url.data_slice() is not None:
-            raise ValueError(
-                "Data slices are not managed. Data path should "
-                "point to a bliss node (h5py.Group)"
-            )
-        self._url = url
-        self._file_handler = None
-
-    def __exit__(self, *exc):
-        return self._file_handler.close()
-
-
-class EntryReader(_BaseReader):
-    """Context manager used to read a bliss node"""
-
-    def __enter__(self):
-        self._file_handler = HDF5File(filename=self._url.file_path(), mode="r")
-        if self._url.data_path() == "":
-            entry = self._file_handler
-        else:
-            entry = self._file_handler[self._url.data_path()]
-        if not isinstance(entry, h5py.Group):
-            raise ValueError("Data path should point to a bliss node (h5py.Group)")
-        return entry
-
-
-class DatasetReader(_BaseReader):
-    """Context manager used to read a bliss node"""
-
-    def __enter__(self):
-        self._file_handler = HDF5File(filename=self._url.file_path(), mode="r")
-        entry = self._file_handler[self._url.data_path()]
-        if not isinstance(entry, h5py.Dataset):
-            raise ValueError(
-                "Data path ({}) should point to a dataset (h5py.Dataset)".format(
-                    self._url.path()
-                )
-            )
-        return entry
+def _ask_for_file_removal(file_path):
+    res = input("Overwrite %s ? (Y/n)" % file_path)
+    return res == "Y"
 
 
 class BaseAcquisition:
@@ -147,6 +88,10 @@ class BaseAcquisition:
 
     _FOV_PATH = "technique/scan/field_of_view"
 
+    _NB_LOOP_PATH = "technique/scan/nb_loop"
+
+    _NB_TOMO_PATH = "technique/scan/nb_tomo"
+
     _TOMO_N_PATH = "technique/scan/tomo_n"
 
     _START_TIME_PATH = "start_time"
@@ -164,6 +109,7 @@ class BaseAcquisition:
         root_url: typing.Union[DataUrl, None],
         configuration: TomoHDF5Config,
         detector_sel_callback,
+        start_index: int,
     ):
         self._root_url = root_url
         self._detector_sel_callback = detector_sel_callback
@@ -179,21 +125,141 @@ class BaseAcquisition:
         self._plugins_pos_resources = {}
         self._plugins_instr_resources = {}
         self._configuration = configuration
+        self._start_index = start_index
 
     @property
     def configuration(self):
         return self._configuration
 
+    @property
+    def start_index(self) -> int:
+        return self._start_index
+
     def write_as_nxtomo(
         self,
-        output_file: str,
-        data_path: str,
+        shift_entry: int,
         input_file_path: str,
         request_input: bool,
         plugins,
+        divide_into_sub_files,
         input_callback=None,
-    ) -> None:
-        raise NotImplementedError("Abstract class")
+    ) -> tuple:
+        self.set_plugins(plugins)
+        nx_tomos = self.to_NXtomos(
+            request_input=request_input,
+            input_callback=input_callback,
+            check_tomo_n=True,
+        )
+
+        # preprocessing to define output file name
+        possible_extensions = (".hdf5", ".h5", ".nx", ".nexus")
+        output_file_basename = os.path.basename(self.configuration.output_file)
+        file_extension_ = None
+        for possible_extension in possible_extensions:
+            if output_file_basename.endswith(possible_extension):
+                output_file_basename.rstrip(possible_extension)
+                file_extension_ = possible_extension
+
+        def get_file_name_and_entry(index, divide_sub_files):
+            entry = "entry" + str(index).zfill(4)
+
+            if self.configuration.single_file or not divide_sub_files:
+                en_output_file = self.configuration.output_file
+            else:
+                ext = file_extension_ or self.configuration.file_extension
+                file_name = (
+                    os.path.splitext(output_file_basename)[0]
+                    + "_"
+                    + str(index).zfill(4)
+                    + ext
+                )
+                en_output_file = os.path.join(
+                    os.path.dirname(self.configuration.output_file), file_name
+                )
+
+                if os.path.exists(en_output_file):
+                    if self.configuration.overwrite is True:
+                        _logger.warning(en_output_file + " will be removed")
+                        _logger.info("remove " + en_output_file)
+                        os.remove(en_output_file)
+                    elif _ask_for_file_removal(en_output_file) is False:
+                        _logger.info(
+                            "unable to overwrite {}, exit".format(en_output_file)
+                        )
+                        exit(0)
+                    else:
+                        os.remove(en_output_file)
+            return en_output_file, entry
+
+        result = []
+        for i_nx_tomo, nx_tomo in enumerate(nx_tomos):
+            output_file, data_path = get_file_name_and_entry(
+                (shift_entry + i_nx_tomo), divide_sub_files=divide_into_sub_files
+            )
+            output_file = os.path.abspath(os.path.relpath(output_file, os.getcwd()))
+            output_file = os.path.realpath(output_file)
+
+            # For pcotomo for example the data path is modified in order to handle with the splitting
+            # that does not fit at 100 % with the current API. for now there is
+            # no convenient way to handle this in a better way
+            assert isinstance(nx_tomo, NXtomo)
+            self._update_nxtomo_from_plugins(nx_tomo=nx_tomo)
+            # embed data if requested
+            vs_0 = nx_tomo.instrument.detector.data[0]
+            if nx_tomo.detector_data_is_defined_by_url():
+                new_urls = []
+                for url in nx_tomo.instrument.detector.data:
+                    if self._copy_frames[url.path()]:
+                        created_url = embed_url(
+                            url=url,
+                            output_file=output_file,
+                        )
+                        new_urls.append(created_url)
+                    else:
+                        new_urls.append(url)
+                nx_tomo.instrument.detector.data = new_urls
+            elif nx_tomo.detector_data_is_defined_by_virtual_source():
+                new_vs = []
+                for vs in nx_tomo.instrument.detector.data:
+                    assert isinstance(vs, h5py.VirtualSource)
+                    assert isinstance(vs_0, h5py.VirtualSource)
+                    url = DataUrl(file_path=vs.path, data_path=vs.name, scheme="silx")
+                    if (
+                        url.path() in self._copy_frames
+                        and self._copy_frames[url.path()]
+                    ):
+                        new_url = embed_url(url, output_file=output_file)
+                        n_vs, _, _ = from_data_url_to_virtual_source(new_url)
+                        new_vs.append(n_vs)
+                    else:
+                        new_vs.append(vs)
+                nx_tomo.instrument.detector.data = new_vs
+            # save data
+            nx_tomo.save(file_path=output_file, data_path=data_path)
+            # check rotation angle
+            if nx_tomo.sample.rotation_angle is not None:
+                unique_angles = numpy.unique(nx_tomo.sample.rotation_angle)
+                if len(unique_angles) == 1:
+                    _logger.warning(
+                        f"NXtomo {data_path}@{output_file} seems to have a single value ({unique_angles}) for rotation angle. Seems it fails to find correct path to the rotation angle dataset"
+                    )
+
+            result.append((output_file, data_path))
+        return tuple(result)
+
+    def _update_nxtomo_from_plugins(self, nx_tomo: NXtomo):
+        if len(self._plugins) > 0:
+            _logger.warning(
+                f"{len(self._plugins)} plugin found. Might modify NXtomo from them"
+            )
+        for plugin in self._plugins:
+            _logger.info(f"Apply plugin {plugin.name}")
+            plugin.update_nx_tomo(
+                nx_tomo=nx_tomo,
+            )
+
+    def to_NXtomos(self, request_input, input_callback) -> tuple:
+        raise NotImplementedError("Base class")
 
     @property
     def raise_error_if_issue(self):
@@ -228,6 +294,13 @@ class BaseAcquisition:
     def root_url(self):
         return self._root_url
 
+    def get_expected_nx_tomo(self):
+        """
+        Return the expected number of nxtomo created for this acquisition.
+        This is required to get consistent entry and file name. At lest for automation
+        """
+        raise NotImplementedError("Base class")
+
     def read_entry(self):
         return EntryReader(self._root_url)
 
@@ -352,17 +425,19 @@ class BaseAcquisition:
             with EntryReader(self._root_url) as entry:
                 if self._TECHNIQUE_MOTOR_PATH in entry:
                     try:
-                        motor_and_aliases = h5py_read_dataset(
-                            entry[self._TECHNIQUE_MOTOR_PATH]
+                        rotation_motor = get_dataset_name_from_motor(
+                            motors=h5py_read_dataset(
+                                numpy.asarray(entry[self._TECHNIQUE_MOTOR_PATH])
+                            ),
+                            motor_name="rotation",
                         )
-                        rotation_motor = motor_and_aliases[0]
                     except Exception as e:
                         _logger.error(e)
                     else:
                         return rotation_motor
                 else:
                     _logger.warning(
-                        "{} does not exists in {}".format(
+                        "{} unable to find rotation motor from {}".format(
                             self._TECHNIQUE_MOTOR_PATH, self._root_url
                         )
                     )
@@ -525,7 +600,7 @@ class BaseAcquisition:
                     info_retrieve=resource_name,
                     expected_unit=None,
                 )
-            except (ValueError, KeyError) as e:
+            except (ValueError, KeyError):
                 pass
             else:
                 return values, None
@@ -556,30 +631,6 @@ class BaseAcquisition:
             nodes=(self._get_instrument_node(root_node), root_node),
         )
 
-    def _ignore_sample_output(self, output_dataset_name):
-        """Should will ignore management of this dataset. Can be the case
-        if managed by a plugin"""
-        for plugin in self._plugins:
-            if output_dataset_name in plugin.sample_datasets_created:
-                return True
-        return False
-
-    def _ignore_detector_output(self, output_dataset_name):
-        """Should will ignore management of this dataset. Can be the case
-        if managed by a plugin"""
-        for plugin in self._plugins:
-            if output_dataset_name in plugin.detector_datasets_created:
-                return True
-        return False
-
-    def _ignore_beam_output(self, output_dataset_name):
-        """Should will ignore management of this dataset. Can be the case
-        if managed by a plugin"""
-        for plugin in self._plugins:
-            if output_dataset_name in plugin.beam_datasets_created:
-                return True
-        return False
-
     def set_plugins(self, plugins):
         """
 
@@ -607,3 +658,21 @@ class BaseAcquisition:
             return "NXTomo"
         else:
             return self.root_url.path()
+
+
+def get_dataset_name_from_motor(motors, motor_name):
+    motors = numpy.asarray(motors)
+    indexes = numpy.where(motors == motor_name)[0]
+    if len(indexes) == 0:
+        return None
+    elif len(indexes) == 1:
+        index = indexes[0]
+        index_dataset_id = index + 1
+        if index_dataset_id < len(motors):
+            return motors[index_dataset_id]
+        else:
+            raise ValueError(
+                f"{motor_name} found but unable to find dataset name from {motors}"
+            )
+    else:
+        raise ValueError(f"More than one instance of {motor_name} as been found.")
