@@ -31,28 +31,25 @@ __authors__ = [
     "H. Payno",
 ]
 __license__ = "MIT"
-__date__ = "27/11/2020"
+__date__ = "14/02/2022"
 
 
 from .baseacquisition import BaseAcquisition
-from .baseacquisition import SourceType
+from nxtomomill.nexus.nxsource import SourceType
 from nxtomomill.io.acquisitionstep import AcquisitionStep
 from .baseacquisition import EntryReader
-from .baseacquisition import DatasetReader
 from .utils import get_entry_type
 from .utils import guess_nx_detector
 from .utils import get_nx_detectors
 from nxtomomill.utils import ImageKey
-from nxtomomill.converter.version import version as converter_version
 from tomoscan.unitsystem import metricsystem
-from tomoscan.io import HDF5File
 from silx.io.utils import h5py_read_dataset
 import h5py
 from silx.io.url import DataUrl
-from typing import Union
+from typing import Optional, Union
 
 try:
-    import hdf5plugin
+    import hdf5plugin  # noqa F401
 except ImportError:
     pass
 import logging
@@ -60,8 +57,7 @@ import fnmatch
 import numpy
 import os
 from nxtomomill.io.config import TomoHDF5Config
-from nxtomomill.converter.utils import create_nx_data_group
-from nxtomomill.converter.utils import link_nxbeam_to_root
+from nxtomomill.nexus.nxtomo import NXtomo
 
 
 _logger = logging.getLogger(__name__)
@@ -69,11 +65,15 @@ _logger = logging.getLogger(__name__)
 
 class StandardAcquisition(BaseAcquisition):
     """
-    A standard acquisition where all registered scan are connected to
-    group an NXTomo entry
+    Class to collect information from a bliss - hdf scan (see https://bliss.gitlab-pages.esrf.fr/fscan).
+    Once all data is collected a set of NXtomo will be created.
+    Those NXtomo instances will then be pass to the plugins to allow users to modify.
+    Then NXtomo instances will be saved to disk.
 
     :param DataUrl root_url: url of the acquisition. Can be None if
                              this is the initialization entry
+    :param TomoHDF5Config configuration: configuration to use to collect raw data and generate outputs
+    :param Optional[Function] detector_sel_callback: possible callback to retrieve missing information
     """
 
     def __init__(
@@ -81,19 +81,16 @@ class StandardAcquisition(BaseAcquisition):
         root_url: Union[DataUrl, None],
         configuration: TomoHDF5Config,
         detector_sel_callback,
+        start_index,
     ):
         super().__init__(
             root_url=root_url,
             configuration=configuration,
             detector_sel_callback=detector_sel_callback,
+            start_index=start_index,
         )
-        # variables set by the `_preprocess_frames` function
-        self._data = None
-        """frames as a virtual dataset"""
-        self._image_key = None
-        """list of image keys"""
+        self._nx_tomos = [NXtomo("/")]
         self._image_key_control = None
-        """list of image keys"""
         self._rotation_angle = None
         """list of rotation angles"""
         self._x_translation = None
@@ -101,20 +98,19 @@ class StandardAcquisition(BaseAcquisition):
         self._y_translation = None
         """y_translation"""
         self._z_translation = None
-        """z_translation"""
-        self._n_frames = None
-        self._dim_1 = None
-        self._dim_2 = None
-        self._data_type = None
+
+        # self._n_frames = None
+        # self._dim_1 = None
+        # self._dim_2 = None
+        # self._data_type = None
         self._virtual_sources = None
         self._acq_expo_time = None
         self._copied_dataset = {}
         "register dataset copied. Key if the original location as" "DataUrl.path. Value is the DataUrl it has been moved to"
-        self._current_scan_n_frame = None
+        # self._current_scan_n_frame = None
 
-    @property
-    def image_key(self):
-        return self._image_key
+    def get_expected_nx_tomo(self):
+        return 1
 
     @property
     def image_key_control(self):
@@ -179,7 +175,12 @@ class StandardAcquisition(BaseAcquisition):
     def is_different_sequence(self, entry):
         return True
 
-    def register_step(self, url: DataUrl, entry_type=None, copy_frames=False) -> None:
+    def register_step(
+        self,
+        url: DataUrl,
+        entry_type: Optional[AcquisitionStep] = None,
+        copy_frames=False,
+    ) -> None:
         """
 
         :param DataUrl url: entry to be registered and contained in the
@@ -243,7 +244,6 @@ class StandardAcquisition(BaseAcquisition):
         data_dataset: h5py.Dataset,
         data_name,
         frame_type,
-        output_file,
         entry,
         entry_path,
         camera_dataset_url,
@@ -281,19 +281,16 @@ class StandardAcquisition(BaseAcquisition):
         # Note: for now there is no image_key on the master file
         # should be added later.
         image_key_control = frame_type.to_image_key_control()
-        image_key = frame_type.to_image_key()
         self._image_key_control.extend([image_key_control.value] * n_frame)
-        self._image_key.extend([image_key.value] * n_frame)
-        # create virtual source (getting ready for writing)
-        rel_input = os.path.relpath(
-            camera_dataset_url.file_path(), os.path.dirname(output_file)
-        )
+
         data_dataset_path = data_dataset.name.replace(entry.name, entry_path, 1)
         # replace data_dataset name by the original entry_path.
         # this is a workaround to use the dataset path on the
         # "treated file". Because .name if the name on the 'target'
         # file of the virtual dataset
-        v_source = h5py.VirtualSource(rel_input, data_dataset_path, data_dataset.shape)
+        v_source = h5py.VirtualSource(
+            camera_dataset_url.file_path(), data_dataset_path, data_dataset.shape
+        )
         self._virtual_sources.append(v_source)
         self._virtual_sources_len.append(n_frame)
         return n_frame
@@ -304,7 +301,6 @@ class StandardAcquisition(BaseAcquisition):
         entry,
         frame_type,
         input_file_path,
-        output_file,
         entry_path,
         entry_url,
     ):
@@ -323,63 +319,38 @@ class StandardAcquisition(BaseAcquisition):
             file_path=entry_url.file_path(), data_path=data_name, scheme="silx"
         )
 
-        if self._copy_frames[entry_url.path()]:
-            from nxtomomill.utils import embed_url
-
-            created_url = embed_url(
-                url=camera_dataset_url,
-                output_file=output_file,
-            )
-            self._copied_dataset[camera_dataset_url.path()] = created_url
-            with DatasetReader(created_url) as copied_data_dataset:
-                n_frame = self.__get_data_from_camera(
-                    copied_data_dataset,
-                    data_name=data_name,
-                    frame_type=frame_type,
-                    output_file=output_file,
-                    entry=entry,
-                    entry_path=created_url.data_path(),
-                    camera_dataset_url=created_url,
-                )
-        else:
-            n_frame = self.__get_data_from_camera(
-                data_dataset,
-                data_name=data_name,
-                frame_type=frame_type,
-                output_file=output_file,
-                entry=entry,
-                entry_path=entry_path,
-                camera_dataset_url=camera_dataset_url,
-            )
+        n_frame = self.__get_data_from_camera(
+            data_dataset,
+            data_name=data_name,
+            frame_type=frame_type,
+            entry=entry,
+            entry_path=entry_path,
+            camera_dataset_url=camera_dataset_url,
+        )
+        # save information if this url must be embed / copy or not. Will be used later at nxtomo side
+        self._copy_frames[camera_dataset_url.path()] = self._copy_frames[
+            entry_url.path()
+        ]
 
         # store rotation
-        if not self._ignore_sample_output("rotation_angle"):
-            rots = self._get_rotation_angle(root_node=entry, n_frame=n_frame)[0]
-            self._rotation_angle.extend(rots)
-        else:
-            self._rotation_angle = None
+        rots = self._get_rotation_angle(root_node=entry, n_frame=n_frame)[0]
+        self._rotation_angle.extend(rots)
 
-        if self.require_x_translation and not self._ignore_sample_output(
-            "x_translation"
-        ):
+        if self.require_x_translation:
             self._x_translation.extend(
                 self._get_x_translation(root_node=entry, n_frame=n_frame)[0]
             )
         else:
             self._x_translation = None
 
-        if self.require_y_translation and not self._ignore_sample_output(
-            "y_translation"
-        ):
+        if self.require_y_translation:
             self._y_translation.extend(
                 self._get_y_translation(root_node=entry, n_frame=n_frame)[0]
             )
         else:
             self._y_translation = None
 
-        if self.require_z_translation and not self._ignore_sample_output(
-            "z_translation"
-        ):
+        if self.require_z_translation:
             self._z_translation.extend(
                 self._get_z_translation(root_node=entry, n_frame=n_frame)[0]
             )
@@ -424,7 +395,7 @@ class StandardAcquisition(BaseAcquisition):
                 return True
         return False
 
-    def _preprocess_registered_entry(self, entry_url, type_, output_file):
+    def _preprocess_registered_entry(self, entry_url, type_):
         with EntryReader(entry_url) as entry:
             entry_path = self._entries_o_path[entry_url.path()]
             input_file_path = entry_url.file_path()
@@ -471,6 +442,7 @@ class StandardAcquisition(BaseAcquisition):
                             pass
                         else:
                             self._diode.extend(diode_vals)
+                            self._diode_unit = diode_unit
 
                     if not self.camera_is_valid(key):
                         _logger.debug(f"ignore {key}, not a `valid` camera name")
@@ -483,12 +455,11 @@ class StandardAcquisition(BaseAcquisition):
                             entry=entry,
                             frame_type=type_,
                             input_file_path=input_file_path,
-                            output_file=output_file,
                             entry_path=entry_path,
                             entry_url=entry_url,
                         )
 
-    def _preprocess_registered_entries(self, output_file):
+    def _preprocess_registered_entries(self):
         """parse all frames of the different steps and retrieve data,
         image_key..."""
         self._n_frames = 0
@@ -498,7 +469,6 @@ class StandardAcquisition(BaseAcquisition):
         self._x_translation = []
         self._y_translation = []
         self._z_translation = []
-        self._image_key = []
         self._image_key_control = []
         self._rotation_angle = []
         self._virtual_sources = []
@@ -519,7 +489,7 @@ class StandardAcquisition(BaseAcquisition):
         # list of data virtual source for the virtual dataset
         for entry_url, type_ in self._registered_entries.items():
             url = DataUrl(path=entry_url)
-            self._preprocess_registered_entry(url, type_, output_file=output_file)
+            self._preprocess_registered_entry(url, type_)
 
         if len(self._diode) == 0:
             self._diode = None
@@ -560,203 +530,6 @@ class StandardAcquisition(BaseAcquisition):
                 res[alias] = self.configuration.param_already_defined[alias]
         return res
 
-    def _write_beam(self, root_node, request_input, input_callback):
-        instrument_node = root_node.require_group("instrument")
-        beam_node = instrument_node.require_group("beam")
-
-        already_defined = self.get_already_defined_params(
-            key=TomoHDF5Config.EXTRA_PARAMS_ENERGY_DK
-        )
-        if already_defined is not None:
-            energy = float(already_defined)
-            unit = "kev"
-        else:
-            energy, unit = self._get_energy(
-                ask_if_0=request_input, input_callback=input_callback
-            )
-        if energy is not None:
-            beam_node["incident_energy"] = energy
-            beam_node["incident_energy"].attrs["unit"] = unit
-
-        if "NX_class" not in beam_node.attrs:
-            beam_node.attrs["NX_class"] = "NXbeam"
-
-    def _write_instrument(self, root_node):
-        # handle instrument
-        instrument_node = root_node.require_group("instrument")
-        instrument_node.attrs["NX_class"] = "NXinstrument"
-        instrument_title = self._get_instrument_name()
-        if instrument_title:
-            instrument_node["name"] = instrument_title
-
-        detector_node = instrument_node.require_group("detector")
-        detector_node.attrs["NX_class"] = "NXdetector"
-        # write data
-        if self._virtual_sources is not None:
-            self._create_data_virtual_dataset(detector_node)
-        if self.image_key is not None:
-            detector_node["image_key"] = self.image_key
-        if self.image_key_control is not None:
-            detector_node["image_key_control"] = self.image_key_control
-        if self._acq_expo_time is not None:
-            detector_node["count_time"] = self._acq_expo_time
-            detector_node["count_time"].attrs["unit"] = "s"
-        # write distance
-        already_defined = self.get_already_defined_params(
-            key=TomoHDF5Config.EXTRA_PARAMS_DISTANCE
-        )
-        if already_defined is not None:
-            distance = float(already_defined)
-            unit = "m"
-        else:
-            distance, unit = self._get_distance()
-        if distance is not None:
-            detector_node["distance"] = distance
-            detector_node["distance"].attrs["unit"] = unit
-        # write x and y pixel size
-        # if magnified pixel size is found then we right this value.
-        # otherwise will take pixel size (if found)
-        already_defined = self.get_already_defined_params(
-            key=TomoHDF5Config.EXTRA_PARAMS_X_PIXEL_SIZE_DK
-        )
-        if already_defined is not None:
-            x_pixel_size = float(already_defined)
-            unit = "m"
-        else:
-            x_pixel_size, unit = self._get_pixel_size("x")
-        if x_pixel_size is not None:
-            detector_node["x_pixel_size"] = x_pixel_size
-            detector_node["x_pixel_size"].attrs["unit"] = unit
-
-        already_defined = self.get_already_defined_params(
-            key=TomoHDF5Config.EXTRA_PARAMS_Y_PIXEL_SIZE_DK
-        )
-        if already_defined is not None:
-            y_pixel_size = float(already_defined)
-            unit = "m"
-        else:
-            y_pixel_size, unit = self._get_pixel_size("y")
-        if y_pixel_size is not None:
-            detector_node["y_pixel_size"] = y_pixel_size
-            detector_node["y_pixel_size"].attrs["unit"] = unit
-        # write field of view
-        fov = self._get_field_of_fiew()
-        if fov is not None:
-            detector_node["field_of_view"] = fov
-            if fov.lower() == "half":
-                estimated_cor, unit = self._get_estimated_cor_from_motor(
-                    pixel_size=y_pixel_size
-                )
-                if estimated_cor is not None:
-                    detector_node["estimated_cor_from_motor"] = estimated_cor
-                    detector_node["estimated_cor_from_motor"].attrs["unit"] = unit
-        # write tomo_n
-        tomo_n = self._get_tomo_n()
-        if tomo_n is not None:
-            # save tomo n
-            detector_node["tomo_n"] = tomo_n
-        # write nx_source
-        source_grp = instrument_node.require_group("source")
-        source_name = self._get_source_name()
-        if source_name:
-            source_grp["name"] = source_name
-        source_type = self._get_source_type()
-        if source_type:
-            # as the value is not saved as a standard value at esrf adapt the value to a standard one
-            if "synchrotron" in source_type.lower():
-                source_type = SourceType.SYNCHROTRON_X_RAY_SOURCE.value
-            # drop a warning if the source type is invalid
-            if source_type not in SourceType.values():
-                _logger.warning(
-                    "Source type ({}) is not a 'standard value'".format(source_type)
-                )
-            source_grp["type"] = source_type
-
-        if "NX_class" not in source_grp.attrs:
-            source_grp.attrs["NX_class"] = "NXsource"
-
-    def _create_data_virtual_dataset(self, detector_node):
-        if (
-            self.n_frames is None
-            or self.dim_1 is None
-            or self.dim_2 is None
-            or self.data_type is None
-        ):
-            if self.n_frames is None:
-                _logger.error("unable to get the number of frames")
-            if self.dim_1 is None:
-                _logger.error("unable to get frame dim_1")
-            if self.dim_2 is None:
-                _logger.error("unable to get frame dim_2")
-            if self.data_type is None:
-                _logger.error("unable to get data type")
-            raise ValueError(
-                "Preprocessing could not deduce all information "
-                "for creating the `data` virtual dataset"
-            )
-        layout = h5py.VirtualLayout(
-            shape=(self.n_frames, self.dim_2, self.dim_1), dtype=self.data_type
-        )
-        last = 0
-        for v_source, vs_len in zip(self._virtual_sources, self._virtual_sources_len):
-            layout[last : vs_len + last] = v_source
-            last += vs_len
-
-        detector_node.create_virtual_dataset("data", layout)
-        detector_node["data"].attrs["NX_class"] = "NXdata"
-        detector_node["data"].attrs[
-            "SILX_style/axis_scale_types"
-        ] = self.get_axis_scale_types()
-        detector_node["data"].attrs["interpretation"] = "image"
-
-    def _check_has_metadata(self):
-        if self.root_url is None:
-            raise ValueError(
-                "no initialization entry specify, unable to" "retrieve energy"
-            )
-
-    def _write_sample(self, root_node):
-        sample_node = root_node.create_group("sample")
-        sample_node.attrs["NX_class"] = "NXsample"
-        sample_name = self._get_sample_name()
-        if sample_name:
-            sample_node["name"] = sample_name
-        group_size = self._get_grp_size()
-        if group_size:
-            sample_node.attrs["group_size"] = group_size
-        if self.rotation_angle is not None:
-            sample_node["rotation_angle"] = self.rotation_angle
-            sample_node["rotation_angle"].attrs["unit"] = "degree"
-        if self.require_x_translation and self.x_translation is not None:
-            sample_node["x_translation"] = self.x_translation
-            sample_node["x_translation"].attrs["unit"] = "m"
-        if self.require_y_translation and self.y_translation is not None:
-            sample_node["y_translation"] = self.y_translation
-            sample_node["y_translation"].attrs["unit"] = "m"
-        if self.require_z_translation and self.z_translation is not None:
-            sample_node["z_translation"] = self.z_translation
-            sample_node["z_translation"].attrs["unit"] = "m"
-
-    def _write_diode(self, root_node):
-        assert self.has_diode, "this acquisition does not expect diode"
-        diode_node = root_node.create_group("diode")
-        diode_node.attrs["NX_class"] = "NXdetector"
-        diode_node["data"] = self._diode
-        if self._diode_unit is not None:
-            diode_node["data"].attrs["unit"] = self._diode_unit
-
-    def _write_plugins_output(self, root_node):
-        for plugin in self._plugins:
-            instrument_node = root_node["instrument"]
-            detector_node = instrument_node["detector"]
-            detector_node.attrs["NX_class"] = "NXdetector"
-            plugin.write(
-                root_node=root_node,
-                sample_node=root_node["sample"],
-                detector_node=detector_node,
-                beam_node=root_node["instrument/beam"],
-            )
-
     def _generic_path_getter(self, path, message, level="warning"):
         """
         :param str level: level can be logging.level values : "warning", "error", "info"
@@ -981,86 +754,118 @@ class StandardAcquisition(BaseAcquisition):
             else:
                 return y_rot / pixel_size, "pixels"
 
-    def write_as_nxtomo(
-        self,
-        output_file: str,
-        data_path: str,
-        input_file_path: str,
-        request_input: bool,
-        plugins,
-        input_callback=None,
-    ) -> None:
-        """
-        write the current sequence in an NXtomo like
+    def to_NXtomos(self, request_input, input_callback, check_tomo_n=True) -> tuple:
+        self._preprocess_registered_entries()
 
-        :param str output_file: destination file
-        :param str data_path: path to store the data in the destination file
-        :param str input_file_path: hdf5 source file
-        :param bool request_input: if some entries are missing should we ask
-                                   the user for input
-        :param list plugins: plugins to process
-        :param input_callback: if provided then will call this callback
-                               function with  (missing_entry, desc) instead of
-                               input
-        """
-        _logger.info(
-            "write data of {} to {}".format(self, output_file + "::/" + data_path)
+        nx_tomo = NXtomo("/")
+
+        # 1. root level information
+        # start and end time
+        nx_tomo.start_time = self._get_start_time()
+        nx_tomo.end_time = self._get_end_time()
+
+        # title
+        nx_tomo.title = self._get_dataset_name()
+        # group size
+        nx_tomo.group_size = self._get_grp_size()
+
+        # 2. define beam
+        energy, unit = self._get_user_settable_parameter(
+            param_key=TomoHDF5Config.EXTRA_PARAMS_ENERGY_DK,
+            fallback_fct=self._get_energy,
+            dtype=float,
+            input_callback=input_callback,
+            ask_if_0=request_input,
         )
-        self.set_plugins(plugins)
+        if energy is not None:
+            # TODO: better manamgent of energy ? might be energy.beam or energy.instrument.beam ?
+            nx_tomo.energy = energy
+            nx_tomo.energy.unit = unit
 
-        # work on absolute path. The conversion to relative path and
-        # then to absolute path is a trick in case there is some 'mounted'
-        # directory exposed differently.
-        # Like '/mnt/multipath-shares/tmp_14_days'
-        output_file = os.path.abspath(os.path.relpath(output_file, os.getcwd()))
-        output_file = os.path.realpath(output_file)
+        # 3. define instrument
+        nx_tomo.instrument.detector.data = self._virtual_sources
+        nx_tomo.instrument.detector.image_key_control = self.image_key_control
+        nx_tomo.instrument.detector.count_time = self._acq_expo_time
+        nx_tomo.instrument.detector.count_time.unit = "s"
+        # distance
+        distance, unit = self._get_user_settable_parameter(
+            param_key=TomoHDF5Config.EXTRA_PARAMS_DISTANCE,
+            fallback_fct=self._get_distance,
+            dtype=float,
+        )
+        if distance is not None:
+            nx_tomo.instrument.detector.distance = distance
+            if nx_tomo.instrument.detector.distance is not None:
+                nx_tomo.instrument.detector.distance.unit = unit
+        # x and y pixel size
+        x_pixel_size, unit = self._get_user_settable_parameter(
+            param_key=TomoHDF5Config.EXTRA_PARAMS_X_PIXEL_SIZE_DK,
+            fallback_fct=self._get_pixel_size,
+            dtype=float,
+            axis="x",
+        )
+        nx_tomo.instrument.detector.x_pixel_size = x_pixel_size
+        if unit is not None:
+            nx_tomo.instrument.detector.x_pixel_size.unit = unit
 
-        # first retrieve the data and create some virtual dataset.
-        self._preprocess_registered_entries(output_file=output_file)
-        with HDF5File(output_file, "a") as h5_file:
-            entry = h5_file.require_group(data_path)
-            entry.attrs["NX_class"] = "NXentry"
-            entry.attrs["definition"] = "NXtomo"
-            entry.attrs["version"] = converter_version()
-            entry.attrs["default"] = "data"
-            start_time = self._get_start_time()
-            if start_time is not None:
-                entry["start_time"] = start_time
-            end_time = self._get_end_time()
-            if end_time is not None:
-                entry["end_time"] = end_time
-            dataset_name = self._get_dataset_name()
-            if dataset_name:
-                entry["title"] = dataset_name
-            entry["definition"] = "NXtomo"
+        y_pixel_size, unit = self._get_user_settable_parameter(
+            param_key=TomoHDF5Config.EXTRA_PARAMS_Y_PIXEL_SIZE_DK,
+            fallback_fct=self._get_pixel_size,
+            dtype=float,
+            axis="y",
+        )
+        nx_tomo.instrument.detector.y_pixel_size = y_pixel_size
+        if unit is not None:
+            nx_tomo.instrument.detector.y_pixel_size.unit = unit
 
-            self._write_beam(
-                entry, request_input=request_input, input_callback=input_callback
-            )
-            self._write_instrument(entry)
-            self._write_sample(entry)
-            self._write_plugins_output(entry)
-            if self.has_diode:
-                self._write_diode(entry)
+        fov = self._get_field_of_fiew()
+        if fov is not None:
+            nx_tomo.instrument.detector.field_of_view = fov
+            if fov.lower() == "half":
+                estimated_cor, unit = self._get_estimated_cor_from_motor(
+                    pixel_size=y_pixel_size
+                )
+                if estimated_cor is not None:
+                    nx_tomo.instrument.detector.estimated_cor_from_motor = estimated_cor
 
-        # create nxdata group
-        try:
-            create_nx_data_group(
-                file_path=output_file,
-                entry_path=data_path,
-                axis_scale=self.get_axis_scale_types(),
-            )
-        except Exception as e:
-            if self.raise_error_if_issue:
-                raise e
-            else:
-                _logger.error(
-                    "Fail to create NXdata group. Reason is {}".format(str(e))
+        # define tomo_n
+        nx_tomo.instrument.detector.tomo_n = self._get_tomo_n()
+
+        # 3. define nx source
+        source_name = self._get_source_name()
+        nx_tomo.instrument.source.name = source_name
+        source_type = self._get_source_type()
+        if source_type is not None:
+            if "synchrotron" in source_type.lower():
+                source_type = SourceType.SYNCHROTRON_X_RAY_SOURCE.value
+            # drop a warning if the source type is invalid
+            if source_type not in SourceType.values():
+                _logger.warning(
+                    "Source type ({}) is not a 'standard value'".format(source_type)
                 )
 
-        # create beam group at root for compatibility
-        link_nxbeam_to_root(file_path=output_file, entry_path=data_path)
+        nx_tomo.instrument.source.type = source_type
 
+        # 4. define sample
+        nx_tomo.sample.name = self._get_sample_name()
+        nx_tomo.sample.rotation_angle = self.rotation_angle
+        nx_tomo.sample.x_translation.value = self.x_translation
+        nx_tomo.sample.x_translation.unit = "m"
+        nx_tomo.sample.y_translation.value = self.y_translation
+        nx_tomo.sample.y_translation.unit = "m"
+        nx_tomo.sample.z_translation.value = self.z_translation
+        nx_tomo.sample.z_translation.unit = "m"
+
+        # 5. define diode
+        if self.has_diode:
+            nx_tomo.instrument.diode.data = self._diode
+            nx_tomo.instrument.diode.data.unit = self._diode_unit
+
+        if check_tomo_n:
+            self.check_tomo_n()
+        return (nx_tomo,)
+
+    def check_tomo_n(self):
         # check scan is complete
         tomo_n = self._get_tomo_n()
         if self.configuration.check_tomo_n and tomo_n is not None:
@@ -1077,3 +882,31 @@ class StandardAcquisition(BaseAcquisition):
                     raise ValueError(mess)
                 else:
                     _logger.error(mess)
+
+    def _check_has_metadata(self):
+        if self.root_url is None:
+            raise ValueError(
+                "no initialization entry specify, unable to" "retrieve energy"
+            )
+
+    def _get_user_settable_parameter(
+        self,
+        param_key,
+        fallback_fct,
+        dtype: Optional[type] = None,
+        *fallback_args,
+        **fallback_kwargs,
+    ):
+        """
+        return value, unit
+        """
+        value = self.get_already_defined_params(param_key)
+        if value is not None:
+            unit = TomoHDF5Config.get_extra_params_default_unit(param_key)
+        else:
+            value, unit = fallback_fct(*fallback_args, **fallback_kwargs)
+
+        if dtype is None or value is None:
+            return value, unit
+        else:
+            return dtype(value), unit
