@@ -34,10 +34,15 @@ __license__ = "MIT"
 __date__ = "27/11/2020"
 
 
+from nxtomomill.converter.hdf5.acquisition.pcotomoacquisition import PCOTomoAcquisition
+from nxtomomill.utils.hdf5 import EntryReader
 from .acquisition.utils import get_entry_type
 from nxtomomill.io.acquisitionstep import AcquisitionStep
 from .acquisition.standardacquisition import StandardAcquisition
-from .acquisition.zseriesacquisition import ZSeriesBaseAcquisition
+from .acquisition.zseriesacquisition import (
+    ZSeriesBaseAcquisition,
+    is_pcotomo_frm_titles,
+)
 from .acquisition.zseriesacquisition import is_z_series_frm_titles
 from .acquisition.zseriesacquisition import is_z_series_frm_z_translation
 from .acquisition.xrdctacquisition import XRDCTAcquisition
@@ -47,6 +52,7 @@ from silx.utils.deprecation import deprecated
 from nxtomomill.io.config import TomoHDF5Config
 from nxtomomill.io.framegroup import FrameGroup
 from nxtomomill.converter.baseconverter import BaseConverter
+from nxtomomill.converter.hdf5.acquisition.baseacquisition import _ask_for_file_removal
 from silx.io.url import DataUrl
 from tomoscan.io import HDF5File
 from typing import Union
@@ -60,10 +66,10 @@ import os
 import h5py
 
 try:
-    import hdf5plugin
+    import hdf5plugin  # noqa F401
 except ImportError:
     pass
-## import that should be removed when h5_to_nx will be removed
+# import that should be removed when h5_to_nx will be removed
 from nxtomomill.converter.hdf5.utils import H5FileKeys
 from nxtomomill.converter.hdf5.utils import H5ScanTitles
 
@@ -80,6 +86,7 @@ H5_X_PIXEL_SIZE = Tomo.H5.X_PIXEL_SIZE
 H5_Y_PIXEL_SIZE = Tomo.H5.Y_PIXEL_SIZE
 H5_DARK_TITLES = Tomo.H5.DARK_TITLES
 H5_INIT_TITLES = Tomo.H5.INIT_TITLES
+H5_PCOTOMO_INIT_TITLES = Tomo.H5.PCOTOMO_INIT_TITLES
 H5_ZSERIE_INIT_TITLES = Tomo.H5.ZSERIE_INIT_TITLES
 H5_PROJ_TITLES = Tomo.H5.PROJ_TITLES
 H5_FLAT_TITLES = Tomo.H5.FLAT_TITLES
@@ -90,6 +97,7 @@ H5_DIODE_KEYS = Tomo.H5.DIODE_KEYS
 DEFAULT_SCAN_TITLES = H5ScanTitles(
     H5_INIT_TITLES,
     H5_ZSERIE_INIT_TITLES,
+    H5_PCOTOMO_INIT_TITLES,
     H5_DARK_TITLES,
     H5_FLAT_TITLES,
     H5_PROJ_TITLES,
@@ -114,11 +122,6 @@ DEFAULT_H5_KEYS = H5FileKeys(
 _logger = logging.getLogger(__name__)
 
 
-def _ask_for_file_removal(file_path):
-    res = input("Overwrite %s ? (Y/n)" % file_path)
-    return res == "Y"
-
-
 class _H5ToNxConverter(BaseConverter):
     """
     Class used to convert a HDF5Config to one or several NXTomoEntry.
@@ -156,7 +159,6 @@ class _H5ToNxConverter(BaseConverter):
                 "configuration should be an instance of HDFConfig"
                 " not {}".format(type(configuration))
             )
-
         self._configuration = configuration
         self._progress = progress
         self._input_callback = input_callback
@@ -266,9 +268,16 @@ class _H5ToNxConverter(BaseConverter):
 
         # step 2: treat FrameGroups
         root_acquisition = None
+        start_index = 0
+        require_pcotomo_expected_nx_tomo = False
         for frame_grp in data_frame_grps:
             if frame_grp.frame_type is AcquisitionStep.INITIALIZATION:
                 current_format = self.configuration.format
+                if require_pcotomo_expected_nx_tomo is True:
+                    _logger.warning(
+                        f"Fail to retrieve expected number of nxtomo for {root_acquisition}"
+                    )
+                require_pcotomo_expected_nx_tomo = False
                 if current_format is Format.STANDARD:
                     from nxtomomill.io.framegroup import filter_acqui_frame_type
 
@@ -288,26 +297,44 @@ class _H5ToNxConverter(BaseConverter):
                             root_url=frame_grp.url,
                             configuration=self.configuration,
                             detector_sel_callback=self.detector_sel_callback,
+                            start_index=start_index,
                         )
+                    elif is_pcotomo_frm_titles(acqui_projs_urls, self.configuration):
+                        root_acquisition = PCOTomoAcquisition(
+                            root_url=frame_grp.url,
+                            configuration=self.configuration,
+                            detector_sel_callback=self.detector_sel_callback,
+                            start_index=start_index,
+                        )
+                        start_index += 0
+                        # this will be defined with the projections
+                        self._require_pcotomo_expected_nx_tomo = True
+
                     else:
                         root_acquisition = StandardAcquisition(
                             root_url=frame_grp.url,
                             configuration=self.configuration,
                             detector_sel_callback=self.detector_sel_callback,
+                            start_index=start_index,
                         )
+                        start_index += root_acquisition.get_expected_nx_tomo()
                 elif current_format is Format.XRD_CT:
                     root_acquisition = XRDCTAcquisition(
                         root_url=frame_grp.url,
                         configuration=self.configuration,
                         detector_sel_callback=self.detector_sel_callback,
                         copy_frames=frame_grp.copy,
+                        start_index=start_index,
                     )
+                    start_index += root_acquisition.get_expected_nx_tomo()
                 elif current_format is Format.XRD_3D:
                     root_acquisition = XRD3DAcquisition(
                         root_url=frame_grp.url,
                         configuration=self.configuration,
                         detector_sel_callback=self.detector_sel_callback,
+                        start_index=start_index,
                     )
+                    start_index += root_acquisition.get_expected_nx_tomo()
                 else:
                     raise ValueError("Format {} is not handled".format(current_format))
                 self.acquisitions.append(root_acquisition)
@@ -321,6 +348,22 @@ class _H5ToNxConverter(BaseConverter):
                     copy_frames=frame_grp.copy,
                 )
 
+                # in case of z we append an index according to if
+                # is already registered or not
+                if isinstance(root_acquisition, ZSeriesBaseAcquisition):
+                    with EntryReader(frame_grp.url) as entry:
+                        z = root_acquisition.get_z(entry)
+                        if z not in self._acquisitions:
+                            start_index += 1
+
+                if require_pcotomo_expected_nx_tomo:
+                    if frame_grp.frame_type is AcquisitionStep.PROJECTION:
+                        nb_loop = root_acquisition.get_nb_loop(frame_grp.url)
+                        nb_tomo = root_acquisition.get_nb_tomo(frame_grp.url)
+                        if nb_loop is not None and nb_tomo is not None:
+                            start_index += int(nb_loop) * int(nb_tomo)
+                            require_pcotomo_expected_nx_tomo = False
+
     def build_acquisition_classes_frm_titles(self):
         """
         Build Acquisition classes that will be used for conversion.
@@ -362,6 +405,8 @@ class _H5ToNxConverter(BaseConverter):
             # list of acquisitions. Once process each of those acquisition will
             # create one 'scan'
             current_acquisition = None
+            start_index = 0
+            require_pcotomo_expected_nx_tomo = False
             for group_name in groups:
                 _logger.debug("parse {}".format(group_name))
                 entry = h5d[group_name]
@@ -386,6 +431,11 @@ class _H5ToNxConverter(BaseConverter):
                     self.configuration.format = "standard"
 
                 entry_type = get_entry_type(url=url, configuration=self.configuration)
+                if entry_type is AcquisitionStep.INITIALIZATION:
+                    if require_pcotomo_expected_nx_tomo is True:
+                        _logger.warning(
+                            f"Fail to retrieve expected number of nxtomo for {current_acquisition}"
+                        )
 
                 # Handle XRD-CT dataset
                 if self.configuration.is_xrdc_ct:
@@ -408,8 +458,11 @@ class _H5ToNxConverter(BaseConverter):
                             root_url=url,
                             configuration=self.configuration,
                             detector_sel_callback=self.detector_sel_callback,
+                            start_index=start_index,
                         )
+                        start_index += current_acquisition.get_expected_nx_tomo()
                         acquisitions.append(current_acquisition)
+
                     elif self._ignore_entry_frm_titles(group_name):
                         current_acquisition = None
                     elif not self._ignore_sub_entry(url):
@@ -438,7 +491,9 @@ class _H5ToNxConverter(BaseConverter):
                             root_url=url,
                             configuration=self.configuration,
                             detector_sel_callback=self.detector_sel_callback,
+                            start_index=start_index,
                         )
+                        start_index += current_acquisition.get_expected_nx_tomo()
                         acquisitions.append(current_acquisition)
                         current_acquisition.register_step(
                             url=url,
@@ -457,25 +512,46 @@ class _H5ToNxConverter(BaseConverter):
                         _logger.warning("ignore entry {}".format(entry))
                 # Handle "standard" tomo dataset
                 elif entry_type is AcquisitionStep.INITIALIZATION:
+                    try:
+                        if is_z_series_frm_titles(
+                            entry=entry, configuration=self.configuration
+                        ):
+                            current_acquisition = ZSeriesBaseAcquisition(
+                                root_url=url,
+                                configuration=self.configuration,
+                                detector_sel_callback=self.detector_sel_callback,
+                                start_index=start_index,
+                            )
+                            start_index += current_acquisition.get_expected_nx_tomo()
+                        elif is_pcotomo_frm_titles(
+                            entry=entry, configuration=self.configuration
+                        ):
+                            current_acquisition = PCOTomoAcquisition(
+                                root_url=url,
+                                configuration=self.configuration,
+                                detector_sel_callback=self.detector_sel_callback,
+                                start_index=start_index,
+                            )
+                            start_index += 0
+                            # this will be defined with the projections
+                            self._require_pcotomo_expected_nx_tomo = True
+                        else:
+                            current_acquisition = StandardAcquisition(
+                                root_url=url,
+                                configuration=self.configuration,
+                                detector_sel_callback=self.detector_sel_callback,
+                                start_index=start_index,
+                            )
+                            start_index += current_acquisition.get_expected_nx_tomo()
+                    except Exception as e:
+                        if self._ignore_entry_frm_titles(group_name):
+                            continue
+                        else:
+                            raise e
                     if self._ignore_entry_frm_titles(group_name):
                         current_acquisition = None
                         continue
 
-                    if is_z_series_frm_titles(
-                        entry=entry, configuration=self.configuration
-                    ):
-                        current_acquisition = ZSeriesBaseAcquisition(
-                            root_url=url,
-                            configuration=self.configuration,
-                            detector_sel_callback=self.detector_sel_callback,
-                        )
-                    else:
-                        current_acquisition = StandardAcquisition(
-                            root_url=url,
-                            configuration=self.configuration,
-                            detector_sel_callback=self.detector_sel_callback,
-                        )
-
                     acquisitions.append(current_acquisition)
                 # continue "standard" tomo dataset handling
                 elif current_acquisition is not None and not self._ignore_sub_entry(
@@ -486,6 +562,22 @@ class _H5ToNxConverter(BaseConverter):
                         entry_type=entry_type,
                         copy_frames=self.configuration.default_copy_behavior,
                     )
+                    # in case of z we append an index according to if
+                    # is already registered or not
+                    if isinstance(current_acquisition, ZSeriesBaseAcquisition):
+                        with EntryReader(url) as entry:
+                            z = current_acquisition.get_z(entry)
+                            if z not in self._acquisitions:
+                                start_index += start_index
+
+                    if require_pcotomo_expected_nx_tomo:
+                        if entry_type is AcquisitionStep.PROJECTION:
+                            nb_loop = current_acquisition.get_nb_loop(url)
+                            nb_tomo = current_acquisition.get_nb_tomo(url)
+                            if nb_loop is not None and nb_tomo is not None:
+                                start_index += int(nb_loop) * int(nb_tomo)
+                                require_pcotomo_expected_nx_tomo = False
+
                 else:
                     _logger.info("ignore entry {}".format(entry))
                 if self.progress is not None:
@@ -528,13 +620,6 @@ class _H5ToNxConverter(BaseConverter):
 
     def write(self):
         res = []
-        possible_extensions = (".hdf5", ".h5", ".nx", ".nexus")
-        output_file_basename = os.path.basename(self.configuration.output_file)
-        file_extension_ = None
-        for possible_extension in possible_extensions:
-            if output_file_basename.endswith(possible_extension):
-                output_file_basename.rstrip(possible_extension)
-                file_extension_ = possible_extension
 
         acq_str = [str(acq) for acq in self.acquisitions]
         acq_str.insert(
@@ -547,64 +632,29 @@ class _H5ToNxConverter(BaseConverter):
                 "init (zserie) titles have been found. You can "
                 "provide more."
             )
-        # step 2: check validity of all the acquisition sequence (consistency)
-        # or write output
+
         if self.progress is not None:
             self.progress.set_name("write sequences")
             self.progress.set_max_advancement(len(self.acquisitions))
-        for i_acquisition, acquisition in enumerate(self.acquisitions):
+
+        # write nx_tomo per acquisition
+        divide_into_sub_files = self.configuration.bam_single_file or not (
+            self.configuration.single_file is False and len(self.acquisitions) == 1
+        )
+
+        for acquisition in self.acquisitions:
             if self._ignore_sub_entry(acquisition.root_url):
                 continue
-            entry = "entry" + str(i_acquisition).zfill(4)
-            if self.configuration.single_file or len(self.acquisitions) == 1:
-                en_output_file = self.configuration.output_file
-            else:
-                ext = file_extension_ or self.configuration.file_extension
-                file_name = (
-                    output_file_basename + "_" + str(i_acquisition).zfill(4) + ext
-                )
-                en_output_file = os.path.join(
-                    os.path.dirname(self.configuration.output_file), file_name
-                )
 
-                if os.path.exists(en_output_file):
-                    if self.configuration.overwrite is False:
-                        _logger.warning(en_output_file + " will be removed")
-                        _logger.info("remove " + en_output_file)
-                        os.remove(en_output_file)
-                    elif _ask_for_file_removal(en_output_file) is False:
-                        _logger.info(
-                            "unable to overwrite {}, exit".format(en_output_file)
-                        )
-                        exit(0)
-                    else:
-                        os.remove(en_output_file)
-            # if acquisition.root_url is None:
-            #     continue
             try:
-                acquisition.write_as_nxtomo(
-                    output_file=en_output_file,
-                    data_path=entry,
+                new_entries = acquisition.write_as_nxtomo(
+                    shift_entry=acquisition.start_index,
                     input_file_path=self.configuration.input_file,
                     request_input=self.configuration.request_input,
                     input_callback=self.input_callback,
                     plugins=self.plugins,
+                    divide_into_sub_files=divide_into_sub_files,
                 )
-                # if split files create a master file with link to those entries
-                if (
-                    self.configuration.single_file is False
-                    and len(self.acquisitions) > 1
-                ):
-                    _logger.info("create link in %s" % self.configuration.output_file)
-                    with HDF5File(self.configuration.output_file, "a") as master_file:
-                        link_file = os.path.relpath(
-                            en_output_file,
-                            os.path.dirname(self.configuration.output_file),
-                        )
-                        master_file[entry] = h5py.ExternalLink(link_file, entry)
-                    res.append((en_output_file, entry))
-                else:
-                    res.append((en_output_file, entry))
             except Exception as e:
                 if self.configuration.raises_error:
                     raise e
@@ -613,9 +663,22 @@ class _H5ToNxConverter(BaseConverter):
                         "Fails to write %s. Error is %s"
                         % (str(acquisition.root_url), str(e))
                     )
+            else:
+                res.extend(new_entries)
             if self.progress is not None:
                 self.progress.increase_advancement()
 
+        # if we created one file per entry then create a master file with link to those entries
+        if self.configuration.single_file is False and divide_into_sub_files:
+            _logger.info("create link in %s" % self.configuration.output_file)
+            for (en_output_file, entry) in res:
+                with HDF5File(self.configuration.output_file, "a") as master_file:
+                    link_file = os.path.relpath(
+                        en_output_file,
+                        os.path.dirname(self.configuration.output_file),
+                    )
+                    master_file[entry] = h5py.ExternalLink(link_file, entry)
+
         return tuple(res)
 
     def _check_conversion_is_possible(self):
@@ -767,6 +830,7 @@ def h5_to_nx(
     # handle titles keys
     configuration.init_titles = scan_titles.init_titles
     configuration.zserie_init_titles = scan_titles.init_zserie_titles
+    configuration.pcotomo_init_titles = scan_titles.init_pcotomo_titles
     configuration.dark_titles = scan_titles.dark_titles
     configuration.flat_titles = scan_titles.flat_titles
     configuration.projections_titles = scan_titles.proj_titles
