@@ -22,23 +22,21 @@
 # THE SOFTWARE.
 #
 #
-# ###########################################################################*/
+# ###########################################################################
 
 """
 contains the HDF5Config
 """
 
-__authors__ = [
-    "H. Payno",
-]
+__authors__ = ["H. Payno, J.Garriga"]
 __license__ = "MIT"
-__date__ = "23/02/2021"
+__date__ = "08/07/2021"
 
 
 from nxtomomill import settings
 from nxtomomill.utils import FileExtension
 from nxtomomill.utils import Format
-from nxtomomill.utils import FieldOfView
+from nxtomomill.nexus.nxdetector import FieldOfView
 from nxtomomill.io.utils import is_url_path
 from nxtomomill.io.utils import convert_str_to_tuple
 from nxtomomill.io.utils import convert_str_to_frame_grp
@@ -299,6 +297,24 @@ class TomoHDF5Config:
     COMMENTS.update(COMMENTS_FRAME_TYPE_SECTION)
     COMMENTS.update(COMMENTS_EXTRA_PARAMS_SECTION)
 
+    __isfrozen = False
+    # to ease API and avoid setting wrong attributes we 'freeze' the attributes
+    # see https://stackoverflow.com/questions/3603502/prevent-creating-new-attributes-outside-init
+
+    @staticmethod
+    def get_extra_params_default_unit(key) -> str:
+        """return the default unit for the extra parameters that can be defined by the user"""
+        if key in (
+            TomoHDF5Config.EXTRA_PARAMS_DISTANCE,
+            TomoHDF5Config.EXTRA_PARAMS_X_PIXEL_SIZE_DK,
+            TomoHDF5Config.EXTRA_PARAMS_Y_PIXEL_SIZE_DK,
+        ):
+            return "m"
+        elif key in (TomoHDF5Config.EXTRA_PARAMS_ENERGY_DK):
+            return "keV"
+        else:
+            raise ValueError(f"No default unit for {key}")
+
     def __init__(self):
         # general information
         self._output_file = None
@@ -310,6 +326,9 @@ class TomoHDF5Config:
         self._no_input = False
         self._format = Format.STANDARD
         self._single_file = False
+        self._bam_single_file = False
+        # a single file is create by default if there is only one entry per file.
+        # but we can enfore multi-file writing
         self._field_of_view = None
 
         # information regarding keys and paths
@@ -328,6 +347,7 @@ class TomoHDF5Config:
         self._sub_entries_to_ignore = None
         self._init_titles = settings.Tomo.H5.INIT_TITLES
         self._zserie_init_titles = settings.Tomo.H5.ZSERIE_INIT_TITLES
+        self._pcotomo_init_titles = settings.Tomo.H5.PCOTOMO_INIT_TITLES
         self._dark_titles = settings.Tomo.H5.DARK_TITLES
         self._flat_titles = settings.Tomo.H5.FLAT_TITLES
         self._projection_titles = settings.Tomo.H5.PROJ_TITLES
@@ -345,6 +365,11 @@ class TomoHDF5Config:
         # extra options
         self._param_already_defined = {}
 
+        self._set_freeze(True)
+
+    def _set_freeze(self, freeze=True):
+        self.__isfrozen = freeze
+
     @property
     def input_file(self) -> Union[None, str]:
         return self._input_file
@@ -463,6 +488,17 @@ class TomoHDF5Config:
         else:
             self._single_file = single_file
 
+    @property
+    def bam_single_file(self):
+        return self._bam_single_file
+
+    @bam_single_file.setter
+    def bam_single_file(self, bam: bool):
+        if not isinstance(bam, bool):
+            raise TypeError("'bam' should be a boolean")
+        else:
+            self._bam_single_file = bam
+
     @property
     def field_of_view(self) -> Union[None, FieldOfView]:
         return self._field_of_view
@@ -651,6 +687,19 @@ class TomoHDF5Config:
         else:
             self._zserie_init_titles = titles
 
+    @property
+    def pcotomo_init_titles(self) -> Union[None, Iterable]:
+        return self._pcotomo_init_titles
+
+    @pcotomo_init_titles.setter
+    def pcotomo_init_titles(self, titles: Union[None, Iterable]) -> None:
+        if titles is None:
+            self._pcotomo_init_titles = None
+        elif not isinstance(titles, Iterable):
+            raise TypeError("'titles' should be None or an Iterable")
+        else:
+            self._pcotomo_init_titles = titles
+
     @property
     def dark_titles(self) -> Union[None, Iterable]:
         return self._dark_titles
@@ -999,7 +1048,7 @@ class TomoHDF5Config:
     def load_keys_section(self, dict_):
         # handle valid camera names. empty string is consider as a valid value
         valid_camera_names = dict_.get(TomoHDF5Config.VALID_CAMERA_DK, None)
-        if valid_camera_names == "":
+        if valid_camera_names in ("", "none", "None", None):
             valid_camera_names = None
         else:
             valid_camera_names = convert_str_to_tuple(
@@ -1099,7 +1148,7 @@ class TomoHDF5Config:
         ]
         flat_titles_dks.extend(TomoHDF5Config.FLAT_TILES_ALIASES)
         flat_title_key_picked = None
-        ## handle flat titles
+        # handle flat titles
         for alias in flat_titles_dks:
             flat_titles = dict_.get(alias, None)
             if flat_titles is not None:
@@ -1144,11 +1193,7 @@ class TomoHDF5Config:
     def load_extra_params_section(self, dict_):
         for key, value in dict_.items():
             if key in TomoHDF5Config.EXTRA_PARAMS_ENERGY_VALID_KEYS:
-                self._param_already_defined.update(
-                    {
-                        key: value,
-                    }
-                )
+                self._param_already_defined.update({key: value})
             else:
                 _logger.warning("{} is not a key handled".format(key))
 
@@ -1195,11 +1240,21 @@ class TomoHDF5Config:
 
 
 class XRD3DHDF5Config(TomoHDF5Config):
+
+    ROCKING_KEYS_DK = "ROCKING_KEYS"
+
+    TomoHDF5Config.COMMENTS.update(
+        {
+            ROCKING_KEYS_DK: "List of keys to look for in order to find rocking angle",
+        }
+    )
+
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
+        self._set_freeze(False)
         self._rocking_keys = settings.XRD3D.H5.ROCKING_KEYS
-        self._base_tilt_keys = settings.XRD3D.H5.BASE_TILT_KEYS
         self._format = Format.XRD_3D
+        self._set_freeze(True)
 
     @property
     def rocking_keys(self) -> Iterable:
@@ -1219,6 +1274,34 @@ class XRD3DHDF5Config(TomoHDF5Config):
         config_parser.read(file_path, encoding=encoding)
         return XRD3DHDF5Config.from_dict(config_parser)
 
+    def load_keys_3dxrd_section(self, dict_):
+        rocking_keys = dict_.get(XRD3DHDF5Config.ROCKING_KEYS_DK, None)
+        # handle rocking
+        if rocking_keys is not None:
+            rocking_keys = convert_str_to_tuple(rocking_keys, none_if_empty=True)
+            self.rocking_keys = rocking_keys
+
+    def load_from_dict(self, dict_: dict) -> None:
+        """Load the configuration from a dictionary"""
+
+        super().load_from_dict(dict_)
+        # general section
+        if self.KEYS_SECTION_DK in dict_:
+            self.load_keys_3dxrd_section(dict_[self.KEYS_SECTION_DK])
+        else:
+            _logger.error("No {} section found".format(self.KEYS_SECTION_DK))
+
+    def to_dict(self) -> dict:
+        """convert the configuration to a dictionary"""
+        _dict = super().to_dict()
+        _dict[XRD3DHDF5Config.KEYS_SECTION_DK].update(
+            {
+                XRD3DHDF5Config.ROCKING_KEYS_DK: self.rocking_keys,
+            }
+        )
+
+        return _dict
+
     @staticmethod
     def from_dict(dict_: dict):
         """
@@ -1231,17 +1314,6 @@ class XRD3DHDF5Config(TomoHDF5Config):
         config.load_from_dict(dict_)
         return config
 
-    @property
-    def base_tilt_keys(self) -> Iterable:
-        return self._base_tilt_keys
-
-    @base_tilt_keys.setter
-    def base_tilt_keys(self, keys) -> None:
-        if not isinstance(keys, Iterable):
-            raise TypeError("'keys' should be an Iterable")
-        else:
-            self._base_tilt_keys = keys
-
 
 class DXFileConfiguration:
     def __init__(self, input_file: str, output_file: Union[str, None] = None):
@@ -1351,6 +1423,9 @@ class DXFileConfiguration:
         self._pixel_size = pixel_size
 
 
-def generate_default_h5_config() -> dict:
+def generate_default_h5_config(config_3dxrd=False) -> dict:
     """generate a default configuration for converting hdf5 to nx"""
-    return TomoHDF5Config().to_dict()
+    if config_3dxrd:
+        return XRD3DHDF5Config().to_dict()
+    else:
+        return TomoHDF5Config().to_dict()
