@@ -27,9 +27,7 @@
 contains the HDF5ConfigHandler
 """
 
-__authors__ = [
-    "H. Payno",
-]
+__authors__ = ["H. Payno"]
 __license__ = "MIT"
 __date__ = "08/03/2021"
 
@@ -45,7 +43,6 @@ import os
 
 _logger = logging.getLogger(__name__)
 
-
 SETTABLE_PARAMETERS_UNITS = {
     "energy": "kev",
     "x_pixel_size": "m",
@@ -249,6 +246,10 @@ class TomoHDF5ConfigHandler(BaseHDF5ConfigHandler):
                 "zserie_init_titles",
                 self.get_tuple_of_keys_from_cmd,
             ),
+            "init_pcotomo_titles": (
+                "pcotomo_init_titles",
+                self.get_tuple_of_keys_from_cmd,
+            ),
             "dark_titles": ("dark_titles", self.get_tuple_of_keys_from_cmd),
             "flat_titles": ("flat_titles", self.get_tuple_of_keys_from_cmd),
             "proj_titles": ("projections_titles", self.get_tuple_of_keys_from_cmd),
@@ -283,7 +284,7 @@ class TomoHDF5ConfigHandler(BaseHDF5ConfigHandler):
                 "dark_titles",
                 "init_zserie_titles",
                 "init_titles",
-                "init_zserie_titles",
+                "init_pcotomo_titles",
                 "x_pixel_size_key",
                 "y_pixel_size_key",
                 "acq_expo_time_keys",
@@ -316,7 +317,7 @@ class TomoHDF5ConfigHandler(BaseHDF5ConfigHandler):
                 return False
 
 
-class XRD3DHDF5ConfigHandler(BaseHDF5ConfigHandler):
+class XRD3DHDF5ConfigHandler(TomoHDF5ConfigHandler):
     def _create_HDF5_config(self):
         if self.argparse_options.config:
             return XRD3DHDF5Config.from_cfg_file(self.argparse_options.config)
@@ -324,47 +325,10 @@ class XRD3DHDF5ConfigHandler(BaseHDF5ConfigHandler):
             return XRD3DHDF5Config()
 
     def _map_option_to_config_param(self, config, options):
+        config = super()._map_option_to_config_param(config, options)
+
         mapping = {
-            "valid_camera_names": (
-                "valid_camera_names",
-                self.get_tuple_of_keys_from_cmd,
-            ),
-            "overwrite": ("overwrite", self.conv_str_to_bool),
-            "file_extension": ("file_extension", filter_str_def),
-            "single_file": ("single_file", self.conv_str_to_bool),
-            "debug": ("log_level", self.conv_log_level),
-            "entries": ("entries", self.get_tuple_of_keys_from_cmd),
-            "ignore_sub_entries": (
-                "sub_entries_to_ignore",
-                self.get_tuple_of_keys_from_cmd,
-            ),
-            "raises_error": ("raises_error", self.conv_str_to_bool),
-            "field_of_view": ("field_of_view", filter_str_def),
-            "request_input": ("request_input", self.conv_str_to_bool),
-            "x_trans_keys": ("x_trans_keys", self.get_tuple_of_keys_from_cmd),
-            "y_trans_keys": ("y_trans_keys", self.get_tuple_of_keys_from_cmd),
-            "z_trans_keys": ("z_trans_keys", self.get_tuple_of_keys_from_cmd),
-            "rot_angle_keys": ("rotation_angle_keys", self.get_tuple_of_keys_from_cmd),
-            "sample_detector_distance": (
-                "sample_detector_distance",
-                self.get_tuple_of_keys_from_cmd,
-            ),
-            "acq_expo_time_keys": (
-                "exposition_time_keys",
-                self.get_tuple_of_keys_from_cmd,
-            ),
-            "x_pixel_size_key": ("x_pixel_size_paths", self.get_tuple_of_keys_from_cmd),
-            "y_pixel_size_key": ("y_pixel_size_paths", self.get_tuple_of_keys_from_cmd),
-            "init_titles": ("init_titles", self.get_tuple_of_keys_from_cmd),
-            "init_zserie_titles": (
-                "zserie_init_titles",
-                self.get_tuple_of_keys_from_cmd,
-            ),
-            "dark_titles": ("dark_titles", self.get_tuple_of_keys_from_cmd),
-            "flat_titles": ("flat_titles", self.get_tuple_of_keys_from_cmd),
-            "proj_titles": ("projections_titles", self.get_tuple_of_keys_from_cmd),
-            "align_titles": ("alignment_titles", self.get_tuple_of_keys_from_cmd),
-            "set_params": ("param_already_defined", _extract_param_value),
+            "rocking_keys": ("rocking_keys", self.get_tuple_of_keys_from_cmd),
         }
         for argparse_name, (config_name, format_fct) in mapping.items():
             argparse_value = getattr(options, argparse_name)
@@ -374,45 +338,13 @@ class XRD3DHDF5ConfigHandler(BaseHDF5ConfigHandler):
         return config
 
     def _check_argparse_options(self, raise_error):
-        if self.argparse_options is None:
-            err = "No argparse options provided"
-            if raise_error:
-                raise ValueError(err)
-            else:
-                _logger.error(err)
-            return False
 
+        super()._check_argparse_options(raise_error)
         options = self.argparse_options
         if options.config is not None:
             # check no other option are provided
             duplicated_inputs = []
-            for opt in (
-                "set_params",
-                "align_titles",
-                "proj_titles",
-                "flat_titles",
-                "dark_titles",
-                "init_zserie_titles",
-                "init_titles",
-                "init_zserie_titles",
-                "x_pixel_size_key",
-                "y_pixel_size_key",
-                "acq_expo_time_keys",
-                "rot_angle_keys",
-                "valid_camera_names",
-                "z_trans_keys",
-                "y_trans_keys",
-                "x_trans_keys",
-                "request_input",
-                "raises_error",
-                "ignore_sub_entries",
-                "entries",
-                "debug",
-                "overwrite",
-                "single_file",
-                "file_extension",
-                "field_of_view",
-            ):
+            for opt in ("rocking_keys",):
                 if getattr(options, opt):
                     duplicated_inputs.append(opt)
             if len(duplicated_inputs) > 0:
