@@ -65,7 +65,7 @@ class Tomo:
         """Keys used by bliss to store translation"""
 
         Y_ROT_KEY = "instrument/positioners/yrot"
-        """Key used by bliss to store the estimated center of rotation for half 
+        """Key used by bliss to store the estimated center of rotation for half
         acquisition"""
 
         DIODE_KEYS = ("fpico3",)
@@ -74,7 +74,7 @@ class Tomo:
         ACQ_EXPO_TIME_KEYS = ("acq_expo_time",)
 
         INIT_TITLES = (
-            "pcotomo:basic",
+            # "pcotomo:basic",  seems to not be used anymore
             "tomo:basic",
             "tomo:fullturn",
             "sequence_of_scans",
@@ -85,7 +85,10 @@ class Tomo:
         initialization scan"""
 
         ZSERIE_INIT_TITLES = ("tomo:zseries",)
-        """specific titles for zserie. Will extend H5_INIT_TITLES"""
+        """specific titles for zserie. Will extend DEFAULT_SCAN_TITLES"""
+
+        PCOTOMO_INIT_TITLES = ("tomo:pcotomo",)
+        """specific titles for pcotomo. Will extend DEFAULT_SCAN_TITLES"""
 
         DARK_TITLES = ("dark images", "dark")
         """if a scan starts by one of those string then is considered as
@@ -144,6 +147,12 @@ class Tomo:
 
 class XRD3D(Tomo):
     class H5(Tomo.H5):
-        ROCKING_KEYS = ("nfdtx",)
 
-        BASE_TILT_KEYS = ("ffdtilt",)
+        positioners_path = "instrument/positioners/"
+
+        ROT_ANGLE_KEYS = (positioners_path + "diffrz", positioners_path + "rot")
+
+        ROCKING_KEYS = (
+            positioners_path + "diffty",
+            positioners_path + "instrument/positioners/dty",
+        )
