@@ -49,7 +49,7 @@ Thus 2.1.0a3 is hexversion 0x020100a3.
 
 """
 
-from __future__ import absolute_import, print_function, division
+from collections import namedtuple
 
 __authors__ = ["Jérôme Kieffer"]
 __license__ = "MIT"
@@ -76,13 +76,12 @@ RELEASE_LEVEL_VALUE = {
 }
 
 MAJOR = 0
-MINOR = 7
-MICRO = 2
-RELEV = "final"
+MINOR = 8
+MICRO = 1
+RELEV = "final"  # <16
 SERIAL = 0  # <16
-date = __date__
 
-from collections import namedtuple
+date = __date__
 
 _version_info = namedtuple(
     "version_info", ["major", "minor", "micro", "releaselevel", "serial"]
